#include <Servo.h>

//VCC to 5V
//GND to GND
//COMP/TRIG to trigPin
//DAC_OUT to sensorPin

const int trigPin = 5;
const int sensorPin = A0;
const int ledPin = 13;
const int btnPin = 4;
const int headPin = 9;
const int shoulderPin = 10;
const int armPin = 11;

Servo headServo;
Servo shoulderServo;
Servo armServo;
int sensorValue = 0;
int marginDistance = 6;   //This is the amount of cm that someone has to move in order for the sensor to register it as movement. Tweak this value according to your space and build.
int distances[9];

//these should be your default values. You might need to change them if you've set up your servos differently
int down = 180;
int front = 90;

bool isON; 
bool eyesOpen;
int buttonState;
bool check;

const int blinkInterval = 2000; //This is the amount of time that robot will keep its eyes closed for

void setup() 
{
  Serial.begin(9600);
  headServo.attach(headPin);
  shoulderServo.attach(shoulderPin);
  armServo.attach(armPin);
  pinMode(ledPin, OUTPUT);
  pinMode(btnPin, INPUT);
  pinMode(trigPin, OUTPUT);                   // A low pull on pin COMP/TRIG
  digitalWrite(trigPin, HIGH);                // Set to HIGH
  delay(500);
  Serial.println("Init the sensor");

  //reset the shoulder, head and armpositions
  shoulderServo.write(down);
  headServo.write(front);
  armServo.write(front);

  isON = false;
  eyesOpen = true;
  check = false;
}

void loop() //Checks if the button is being pressed. Otherwise, keep switching between eyes open and eyes closed state
{
  buttonState = digitalRead(btnPin);
  if(isON)
  {
    if(eyesOpen)
    {//Start scanning for movement
      openEyes();
    }
    else if (eyesOpen == false)
    {//Head is in neutral position and doesn't scan for movement
       digitalWrite(ledPin, LOW);
       Serial.println("eyes closed");
       for (int i = 0; i<= blinkInterval; i++)
       {
        delay(2);
         checkBtn();
         if(isON == false)
         {
           return;
         }
       }
       eyesOpen = true;
    }
  }
  checkBtn();
}

void checkBtn() //Checks if the button is being pressed. If it's pressed, it toggles the robot on or off
{
  buttonState = digitalRead(btnPin);
  if(buttonState == 1)
  {
    check = true;
  }
  if(buttonState == 0 && check == true)
  {
     check = false;
    Serial.println("ToggledON");
    digitalWrite(ledPin, HIGH);
    delay(100);
    digitalWrite(ledPin, LOW);
      isON = !isON; 
  }
  
}

void openEyes() //Scans whether something has moved
{
  digitalWrite(ledPin, HIGH);
  // rotates the servo motor from 90 to 180 degrees
  //Scan and save left part
  for (int i = 90; i <= 180; i++) 
  {
    checkBtn();
    if(isON == false)
    {
      return;
    }
    headServo.write(i);
    delay(20);
    switch(i)
    {
      case 90:
          distances[4] = readDistance();
          break;
      case 100:
          distances[5] = readDistance();
          break;
      case 120:
          distances[6] = readDistance();
          break;
      case 140:
          distances[7] = readDistance();
          break;
      case 160:
          distances[8] = readDistance();
          break;
    }
  }

  // Repeats the previous lines from 180 to 0 degrees
  //Check with saved left part + scan and save right part
  for (int i = 180; i > 0; i--) 
  {
    checkBtn();
    if(isON == false)
    {
      return;
    }
    headServo.write(i);
    delay(20);

    switch(i)
    {
      case 20:
          distances[0] = readDistance();
          break;
      case 40:
          distances[1] = readDistance();
          break;
      case 60:
          distances[2] = readDistance();
          break;
      case 80:
          distances[3] = readDistance();
          break;
      case 90:
          checkIfTriggered(4, i);
          break;
      case 100:
          checkIfTriggered(5, i);
          break;
      case 120:
          checkIfTriggered(6, i);
          break;
      case 140:
          checkIfTriggered(7, i);
          break;
      case 160:
          checkIfTriggered(8, i);
          break;
    }
  }

  // rotates the servo motor from 0 to 90 degrees
  //check with saves right part
  for (int i = 0; i <= 90; i++) 
  {
    checkBtn();
    if(isON == false)
    {
      return;
    }
    headServo.write(i);
    delay(20);
    
        switch(i)
    {
      case 20:
          checkIfTriggered(0, i);
          break;
      case 40:
          checkIfTriggered(1, i);
          break;
      case 60:
          checkIfTriggered(2, i);
          break;
      case 80:
          checkIfTriggered(3, i);
          break;
    }
  }
  eyesOpen = false;
}

void checkIfTriggered(int dis, int i) //Checks whether the first registered value differs enough from the second scanned value
{
  int tempDistance = readDistance();
  int tempAnswer = tempDistance - distances[dis];
  tempAnswer = abs(tempAnswer);
  if(tempAnswer > marginDistance)
  {
     Serial.print("MOVED AT ");
     Serial.println(i);
     triggeredRobot(i);
  }
}

int readDistance() //code provided with the URM37 V5.0 ultrasonic sensor
{
  digitalWrite(trigPin, LOW);
  digitalWrite(trigPin, HIGH);
  delay(20);
  sensorValue = analogRead(sensorPin);

  sensorValue = sensorValue * 1.1; //  (sensorValue * 5000 / 1024 ) / 4.125 = sensorValue * 1.1 , calculate the voltage value ADC collected and divide it by 4.125mV /cm.
  return (sensorValue);
}

int triggeredRobot(int angle) //robot stops and points to the angle where something has moved. Waits, puts its arm back down and shuts off.
{
  Serial.println(angle);
  int up = 90;
  shoulderServo.write(up);
  delay(500);
  
  //max en min values of the servo, due to physical constrains
  if(angle > 150)
  {
    angle = 150;
  }
  else if(angle < 50)
  {
    angle = 50;
  }
  
  armServo.write(angle);
  delay(5000);
  armServo.write(front);
  delay(500);
  shoulderServo.write(down);
  delay(500);
  headServo.write(front);
  digitalWrite(ledPin, LOW);
  isON = false;
}
